"ellipse" <-
function(XY.mat, alpha=0.95, ell.res=5, ell.lwd=1, ell.type=3,
				p.axis=TRUE, p.centroid=FALSE, centroid.type=20,
				col="black", name=NULL, name.pos=NULL, lwd=1, cex=1) {
	#
	# Draw confidence ellipses around groups of points.
	# Reference: Biometry (Sokal & Rohlf 1995), Section 15.7
	#
	# Confidence regions: values between 0.01 and 0.999
	# ell.res: the default is to produce a point each 5 degrees
	# p.axis: print the ellipse major and minor axes
	# p.centroid: print the centroid
	# ell.type:  type of line used to draw the ellipse and its axis;
	#    see 'lty' in the help documentation file of 'par'
	# centroid.type: type of centroid used to be displayed;
	#    see 'pch' in the help documentation file of 'par'
	# col: the color used for the ellipses;
	#    see 'col' in the help documentation file of 'par'
	# names: names of the ellipse to be plot (near the centroid)
	# name.pos: the position relative to the centroid pos: values of '1', '2', '3'
    #    and '4', respectively, indicate positions below, to the left of, above,  
    #    or to the right of the specified coordinates.
    #
	if(!is.logical(c(p.axis,p.centroid))){
		cat("Choose either 'FALSE' or 'TRUE' for both 'p.axis' and 'p.centroid':","\n")
	}
	step=5
	# Compute the 95% confidence ellipse 95% if alpha = 0.95
	n<-length(XY.mat[,1])
	if(n<=2){
		cat("Not enough individuals in group to draw ", name ,"ellipse\n")
	}else{
		# Variance
		Sx<-var(XY.mat[,1]);Sy<-var(XY.mat[,2])
		# Mean
		Mx<-mean(XY.mat[,1]);My<-mean(XY.mat[,2])
		#print(cbind(Mx,My))
		
		Eig<-eigen(cov(cbind(XY.mat)))$values
		
		# Covariance
		COVxy<-cov(XY.mat)
		Lx=Eig[1]
		if(Lx<0) Lx=0 
		
		Ly=Eig[2]
		if(Ly<0) Ly=0
		# Slopes of major and minor axes
		b1<-COVxy[1,2]/(Lx-Sx)
		if(is.infinite(b1) | is.na(b1)) b1<-0
		b2<--1/b1
		if(is.infinite(b2) | is.na(b1)) b2<-0
	
		C<-((Lx*Ly)*((n-1)*2)/(n-2)*(qf(alpha,2,n-2)))
		
		locc<-sqrt(C/(Ly*(1+b1^2)))
		if(is.infinite(locc) | is.na(locc)) locc<-0
		#e
		a.y<-My+locc
		a.x<-Mx+b1*(a.y-My)
		#f
		b.y<-My-locc
		b.x<-Mx+b1*(b.y-My)
	
		locc<-sqrt((C/(Lx*(1+b2^2))))
		if(is.infinite(locc) | is.na(locc)) locc<-0
		#g
		c.y<-My+locc
		c.x<-Mx+b2*(c.y-My)
		#h
		d.y<-My-locc
		d.x<-Mx+b2*(d.y-My)

		r.grand<-sqrt((a.y-My)^2+(a.x-Mx)^2)
		r.grand<-sqrt((b.y-My)^2+(b.x-Mx)^2)
		r.petit<-sqrt((c.y-My)^2+(c.x-Mx)^2)
		r.petit<-sqrt((d.y-My)^2+(d.x-Mx)^2)
		step=ell.res*pi/180
		angle<-seq(0,(2*pi),by=step)
		El<-matrix(0,length(angle),2)
		El[,1]=r.grand*sin(angle)
		El[,2]=r.petit*cos(angle)
		mat<-eigen(cov(XY.mat))$vectors
		El<-(El%*%t(mat))
		El[,1]<-El[,1]+Mx
		El[,2]<-El[,2]+My
		if(round(var(El[,1]),digit=10)==0 | round(var(El[,2]),digit=10)==0) {
			cat(name,"ellipse was not drawn because the group had its major and/or",
			    "minor axis with length near zero\n")
		}else{
			lines(El[,1],El[,2],lty=ell.type,col=col,lwd=ell.lwd)
		}
		if(p.centroid==TRUE){
			points(x=Mx,y=My,col=col,pch = centroid.type)
		}else{}
		if(p.axis==TRUE){
			lines(x=c(a.x,b.x),y=c(a.y,b.y),lty=ell.type,col=col,lwd=lwd)
			lines(x=c(c.x,d.x),y=c(c.y,d.y),lty=ell.type,col=col,lwd=lwd)
		}else{}
		if(length(names)!=0)
			text(x=Mx,y=My,label=name,col=col,pos=name.pos, cex=cex)
	}
}

