"graph.rdaTest" <-
function(rdaTest, xax=1, yax=2, scaling=1, plot.type="notchosen", binary=NULL,
	groups=NULL, centroids=TRUE, ell=TRUE, sites=TRUE, stars=FALSE,
	label.env=TRUE, label.spc=TRUE, label.site=TRUE,
	alpha=0.95, saveplot=FALSE, path=NULL, width=6, height=6, 
	xlim=NULL, ylim=NULL,lwd=1, len=0.1,
	lty.env=1, lty.spc=1, lty.ell=1, lty.stars=1, lty.axis=2, 
	col.env="blue",col.spc="red", col.groups=NULL, col.ell="black", 	
	col.ctd="black",
	pos.env=NULL, pos.bin=4, pos.site=NULL, pos.ctd=4,
	cex.env=1, cex.spc=1, cex.site=1, cex.ctd=1, cex.axis=1,
	pch.bin=21, pch.site=21, pch.ctd=23, 
	ell.axis=FALSE, mul.spc=0.90, mul.env=0.90, mul.text=0.10,
	mai.perc=NULL)

# Version 1.0

# Function to produce triplots from the output list of function 'rdaTest'
#
# Sbastien Durand, Universit de Montral, April 2005
#
# rdaTest: name of the rdaTest output object. The most simple call to this function
#    is to provide only the 'rdaTest' output object name. Ex.: graph.rdaTest(toto)
#
# The following parameters can also be specified by the user:
#
# xax and yax: the matrix columns to be used to plot in the rda graph.
#    These value can be negative; if so, the plot axes will be reversed
# binary: a vector specifying which of the environmental variables are binary;
#    the corresponding data points will be averaged and represented by a symbol
#    instead of an arrow, as in Canoco
# scaling: allows the user to choose between the two scaling type 1 or 2
# plot.type: Either "F" or "Z" -- "F"= site scores, "Z" = fitted site scores
# saveplot: If you want to save the plot to an .pdf file;
#    if set to FALSE, the plot will only be plotted in a window (quartz);
#    If set to TRUE, no graph will be plotted on screen, and a pdf file will created
# path: complete path to the file in which the plot will be saved; 
#    example: "~/Desktop/toto.pdf"
# width and height: sizes in inches of the drawn plot
# xlim and ylim: vectors describing the minimum and maximum plotted region 
# lwd: line width for the axis, arrow, and ellipses
# len: length of the arrow heads
#
# Parameters related to the confidence ellipses around groups of points:
# ell: draw confidence ellipses around groups of points defined by a vector 
#    defining the group assignments of the objects; see example below
# alpha: confidence region of the ellipses (e.g.: 0.90, 0.80, etc.) Default value: 0.95
# lty.ell: the drawing type; see 'lty' in the help documentation file of 'par'
# col.ell: the ellipses and groupment text color; see 'lty' in the help file of 'par'
# ell.axis: if TRUE, draw the major and minor axes of the ellipses
# pos.bin: offset the names of the binary variables:
#    1=bottom, 2=left, 3=top, 4=right
#
# lty.env: the environmental variables arrow drawing type
# lty.spc: the species variables arrow drawing type
# col.env: the environmental variables arrow and text color
# col.spc: the species variables arrow and text color
# lty.axis: the drawing type of axis, dotted, line, .... 
# text.cex: the font size
# mul.spc: multiplier based on the plot maximum ranges in x and y; 
#    changes proportionally the dimension of species arrows; 
#    1 is the maximum spend of the whole arrow set
# mul.env: the same as Mul but for the environmental variable
# mul.text: plots the text position at the end of each arrow plus a percentage; 
#    default is 10%
# mai.perc:  Allows to increase or decrease the margin size, it uses a percentage of the 
#	   total width and height of the graph, use value between 0 and 1
# label...:  Define using logicals whether the labels shall be plotted


# Use of the function in conjunction with 'rdaTest':
#

{
# Axis reversion filter
	if(saveplot==TRUE){
		if(is.null(path)){
			cat("Type a file name describing this plot: ex.: toto","\n")
			nom<-scan(file="",what="character",quiet=TRUE,nline=1)
			cat("Do you want to save '",nom,
				".pdf' in the working directory: YES='Y' or NO='N'","\n",sep="")
			tmp<-scan(file="",what="character",quiet=TRUE,nline=1)
			if(tmp!="Y"){
				cat("Type the directory path you require ~/.../:","\n")
				tmp1<-scan(file="",what="character",quiet=TRUE,nline=1)
				path<-paste(tmp1,nom,".pdf",sep="")
			}else{
				path<-paste(getwd(),"/",nom,".pdf",sep="")
			}
		}else{}
		pdf(file = path, onefile = TRUE, paper="special", width=width, 
			height=height, family="Helvetica")
	}else{
		dev.size(w=width,h=height)
	}	
	old.par <- par(no.readonly = TRUE) 
	# all par settings which could be changed.
	on.exit(par(old.par))
	
	xinv<-1
	if(xax<0){
		xinv<-(-1)
		xax<--(xax)
	}else{}
	yinv<-1
	if(yax<0){
		yinv<-(-1)
		yax<--(yax)
	}else{}
	if(xax > ncol(rdaTest$F))
		stop("Their are not so many canonical axes; change the xax value")
	if(yax > ncol(rdaTest$F))
		stop("Their are not so many canonical axes, change the yax value")
		
# Scaling 	
	if(scaling!=1){
		rdaTest$F<-rdaTest$FSc2
		rdaTest$Z<-rdaTest$ZSc2
		rdaTest$U<-rdaTest$USc2
		rdaTest$biplotScores1<-rdaTest$biplotScores2
	}else{}
#
	plot.type<-toupper(as.character(plot.type))
	if(plot.type=="notchosen" | plot.type!="F" & plot.type!="Z"){
		while(plot.type!="F" & plot.type!="Z"){
			cat("Choose a functionnal plot.type: type 'Z' for fitted site",
				" scores or 'F' for site scores","\n") 
			plot.type<-toupper(scan(file="", what="character", nlines=1, 		
			quiet=TRUE))
		}
	}else{}
	
	if(plot.type!="Z"){
		rdaTest$Z<-rdaTest$F
	}else{}
	
	if(!is.null(binary)){
		binvar<-matrix(NA,length(binary),ncol(rdaTest$Z))
		j=1
		for(i in binary){
			tmp<-which(rdaTest$X.mat[,i]!=0)
			binvar[j,]<-apply(rdaTest$Z[tmp,]*rdaTest$X.mat[tmp,i],2,mean)
			j=j+1
		}
		colnames(binvar)<-colnames(rdaTest$Z)
		rownames(binvar)<-colnames(rdaTest$X.mat)[binary]
		# rownames(binvar)<-c("Ra","Ri","Rs","Lv","Ln","Ll","Ls","Lf","Sc","Iba","Vl",
		#    "Vp","Vh","Vc","Tw","Tb","Hd","At","As","Pb","Li","Cl","Ch","Cb")
		# rownames(binvar)<-c("a","b","c","d","e","f","g","h","i","j","k","l","m","n",
		#    "o","p","q","r","s","t","u","v","w","x")
	}else{}
	# rownames(rdaTest$U)<-c("1","2","3","4","5","6","7","8","9","10","11","12","13",
	#    "14","15","16","17","18","19","20","21")
	
	# Text adaptation
	mul.text=mul.text+1
	
## Check and select plot.type and application
	
	#cc<-range(xinv*rdaTest$U[,xax]);dd<-range(yinv*rdaTest$U[,yax])
	# type="n"
	#Control the marging size
	if(length(mai.perc)!=0){ 
		par(mai=c(height*mai.perc, width*mai.perc, height*mai.perc/2, 
			width*mai.perc/2))
	}else{}
	if(is.null(groups)){
		groups=as.factor(rep(1,nrow(rdaTest$Z)))
		if(is.null(col.groups)){
			col.groups<-"Black"
		}else{}
	}else{
		groups=as.factor(groups)
		if(is.null(col.groups)){
			col.groups<-palette(topo.colors(length(levels(groups))))
		}else{}
	}
	
	plot(xinv*rdaTest$Z[,xax], yinv*rdaTest$Z[,yax], xlim = xlim, ylim = ylim, 
		asp = 1, cex.axis = cex.axis,
		xlab = paste("Canonical axis ", xax, " ( ", round(rdaTest$AxisVar[xax], 
			dig=2), "% )", sep=""),
		ylab = paste("Canonical axis ", yax, " ( ", round(rdaTest$AxisVar[yax], 
			dig=2), "% )", sep=""),
		type = "n")
	# Print zero axis
	abline(h = 0, lty = lty.axis, lwd=lwd)
	abline(v = 0, lty = lty.axis, lwd=lwd)
	
	mult.spc=multi.factor(rdaTest$Z, rdaTest$U, xax=xax, yax=yax,  percentage=mul.spc)
	mult.env<-multi.factor(rdaTest$Z, rdaTest$biplotScores1, xax=xax, yax=yax, 
percentage=mul.env)

	coll=1
	
	for(i in levels(groups)){
		nn<-which(groups==i)
		if(sites==TRUE){
			if(pch.site>=21 & pch.site<=25){
				points(x=xinv*rdaTest$Z[nn,xax], yinv*rdaTest$Z[nn,yax], 
					pch=pch.site, bg=col.groups[coll], cex=cex.site)
			}else{
				points(x=xinv*rdaTest$Z[nn,xax], yinv*rdaTest$Z[nn,yax], 
					pch=pch.site, col=col.groups[coll], cex=cex.site)	
			}		
			if(label.site==TRUE){
				text(x=xinv*rdaTest$Z[nn,xax], y=yinv*rdaTest$Z[nn,yax], 
					rownames(rdaTest$Z)[nn], col = 'black', 		
					pos=pos.site)
			}else{}
		}else{}
		tmp<-c(NA,NA)
		tmp[1]<-mean(rdaTest$Z[nn,xax])
		tmp[2]<-mean(rdaTest$Z[nn,yax])
		
		if(stars==TRUE){
			arrows(x0=xinv*tmp[1], y0=yinv*tmp[2], x1=xinv*rdaTest$Z[nn,xax], 		
				y1=yinv*rdaTest$Z[nn,yax], length=0, col=col.groups[coll], 	
				lty=lty.stars, lwd=lwd)
		}else{}
		if(centroids==TRUE){
			if(pch.ctd>=21 & pch.ctd<=25){
				points(x=xinv*tmp[1], y=yinv*tmp[2], pch=pch.ctd, 
					bg=col.groups[coll])
			}else{
				points(x=xinv*tmp[1], y=yinv*tmp[2], pch=pch.ctd, 
					col=col.groups[coll])
			}
			text(x=xinv*tmp[1], y=yinv*tmp[2], pos=pos.ctd,
				labels=as.character(levels(groups)[coll]), col=col.ctd, 
				cex=cex.ctd)
		}else{}
	coll=coll+1
	}
	
	
	# Drawing of species variable arrows
	arrows(x0=0, y0=0,  x1=xinv * rdaTest$U[,xax] * mult.spc, 
		y1=yinv*rdaTest$U[,yax] * mult.spc, 
		col=col.spc, code=2, lty=lty.spc, len=len, lwd=1)
	if(label.spc==TRUE)
		text(x=mul.text*mult.spc*xinv*rdaTest$U[,xax], 
			y=mul.text*mult.spc*yinv*rdaTest$U[,yax], 
			rownames(rdaTest$U), col = col.spc, cex = cex.spc)
	
	if(!is.null(binary)){
	# Draw environmental variable points since they are binary variables	
		points(x=xinv*binvar[,xax], y=yinv*binvar[,yax], pch=10, col=col.env)
		if(label.env==TRUE)
			text(x=xinv*binvar[,xax],y=yinv*binvar[,yax], 
				label=rownames(binvar), col=col.env, pos=pos.bin, cex=cex.spc)
		remain<-seq(1, ncol(rdaTest$X.mat),by=1)[-binary]
	}else{
		remain<-seq(1, ncol(rdaTest$X.mat),by=1)	
	}
	if(length(remain)!=0){
		arrows(x0=0, y0=0, xinv*rdaTest$biplotScores1[remain,xax]*mult.env, 
			yinv*rdaTest$biplotScores1[remain,yax]*mult.env, lty=lty.env,
			col=col.env, code=2, len=len, lwd=1)
		if(label.env==TRUE)
			text(x=mul.text*mult.env*xinv*rdaTest$biplotScores1[remain,xax], 
				y=mul.text*mult.env*yinv*rdaTest$biplotScores1[remain,yax], 
				rownames(rdaTest$biplotScores1)[remain], col = col.env, 
				cex=cex.env)
	}else{}	
	# Draw environmental variable arrows	
	len=9
	axis(3, at = seq(-mult.env,mult.env,length=len), 
		labels =  round(seq(-1,1,length=len),digit=2), col = col.env, 
		cex.axis=cex.axis)
	axis(4, at = seq(-mult.env,mult.env,length=len), 
		labels =  round(seq(-1,1,length=len),digit=2), col = col.env,
		cex.axis=cex.axis)
	if(ell==TRUE){
		name.ell=levels(as.factor(groups))
		for(i in name.ell){
			numb<-which(groups==i)
			ellipse(XY.mat=cbind(xinv*rdaTest$Z[numb,xax],
				yinv*rdaTest$Z[numb,yax]), alpha=alpha, ell.type=lty.ell, 
				ell.lwd=lwd,name=as.character(i), p.axis=ell.axis, col=col.ell)
		}
	}else{}
	# Close the drawn plot
	if(saveplot==TRUE)
		dev.off()
}