"ldaTest" <-
function(X.mat,group, print.res=TRUE,alpha=0.05, cross=NULL){

# Function LDA for complete or cross-validation testing.
# Computed as described in Numerical Ecology, Chapter 11.5 (Legendre & Legendre # 1998)
#
# Pierre Legendre, Sbasiten Durand, Universit de Montral, March-April 2005
#
# PARAMETERS:
#
# X.mat (nxm) contains the explanatory variables. The number of variables 'm' is # computed after eliminating collinear explanatory variables, if any
#group
#		(required if no formula principal argument is given.) a factor specifying the class for each observation.
#
# print.res: prints most of ldaTest output on the screen.
#
# The function returns an output list containing the following ELEMENTS:
#
# return(list(VIF, canEigval, C, F=F, Z=Z, FSc2=FSc2, 
#        ZSc2=ZSc2, biplotScores1=posX, biplotScores2=posXSc2, FitSpe=Frac$rdaFitSpe, 
#        VarExpl=Frac$VarExpl, X.mat=X.mat))
#
# VIF: variance inflation factors for the explanatory variables X; 
#    the value is 0 for entirely collinear variables
# canEigval: canonical eigenvalues
# C (pxk): Calculates the C matrix "Final vectors for the explanatory variables"
# F (nxk): matrix of object scores
# Grp.Ctr (kxg): Group centroids
# Homo :  Test for the hypothesis of homogeneity of the within group dispersion matrices
#				Kullback's statistics 
#Class.Ftn	(kxp) : Weights of the canonical functions
#Scores
#Grp.Obs
# Grp.Ass
# Class.Table
#Stats
	
#
# Use of the function 'ldaTest':
#
# First set of examples: use all default values, except the data file names
# result <- ldaTest(X)         # simple example
#
# Second set of examples: where cross-validation is used: 50% to build the model and 50% for evaluation
# result <- ldaTest(X, cross=0.5)
#
#
	
#	library(MASS)
	if(!is.matrix(X.mat))
		stop("'X.mat' must be a matrix\n")
	if(!is.vector(group) & !is.factor(group))
		stop("'group' must be a vector or a factor\n")
	if(is.vector(group))
		group<-as.factor(group)
	g.levels<-(levels(as.factor(group)))
	
	Var.names<-colnames(X.mat)
	if(is.null(Var.names))
		Var.names<-paste("Var", 1:ncol(X.mat),sep="")
	Obj.names<-rownames(X.mat)
	if(is.null(Obj.names))
		Obj.names<-paste("Obj", 1:nrow(X.mat), sep="")

	ldaTest<-list()
	# Identify collinear columns in X and remove them
	ldaTest$VIF=vif(X.mat,print.vif=print.res)
	tmp<-which(ldaTest$VIF==0)
	if(length(tmp)!=0){
		X.cor = X.mat[,-tmp]
		Var.names<-Var.names[-tmp]
	}else{
		X.cor = X.mat
	}
	if(print.res==TRUE){ 
		cat('\nVariance Inflation Factors (VIF)\n')
		print(	ldaTest$VIF)
	}
	
	sel<-NULL
	notsel<-1:nrow(X.cor)
	if(!is.null(cross)){
		if(cross==0 | cross==1){
			cat("Presently only values between ]0,1 [can be used!\n")
			cat("Cross values has been converted to NULL\n")
			cross<-NULL
		}else{}
	}else{}
	if(!is.null(cross)){
		k=0
		for(i in g.levels){
			tmp<-which(group==i)
			tmp0<-sample(tmp,round(length(tmp)*cross),replace=FALSE)
			if(k==0){
				sel<-tmp0
				k=1
			}else{
				sel<-sort(c(sel,tmp0))
			}
			if(length(tmp0)<=1){
				sel=NULL
				stop("Not enough objects remaining in group ",i,
					" after resampling, try another 'cross' value\n")
			}else{}
		}
		notsel<-notsel[-sel]
		if(length(sel)!=0 & length(sel)!=length(group)){
			X.cor.notsel<-X.cor[-sel,]  
			group.notsel<-group[-sel]
			X.cor<-X.cor[sel,]
			group<-group[sel]
		}else{
			sel=NULL
		}
	}else{}
	if(length(sel)==0){
		X.cor.notsel<-X.cor
		group.notsel<-group
		sel=notsel
	}else{}
	# This step is necessairy to allow the explanatory variable arrows to be
	# plotted over the objects position
	Xctr=scale(X.cor,center=TRUE,scale=TRUE)
	X.cor.notsel<-scale(X.cor.notsel,center=TRUE,scale=TRUE)
		
	g.levels<-(levels(as.factor(group)))
	nb.g<-length(g.levels)
	n<-length(sel)
	m=ncol(Xctr)
	centroid<-do.call("cbind", lapply(g.levels,function(y) 
		apply(Xctr[which(group==y),],2,mean)))
	colnames(centroid)<-paste("Group", g.levels)
	rownames(centroid)<-paste(Var.names,".Ctr", sep="")
	print(centroid)
	T<-t(Xctr-mean(Xctr))%*%(Xctr-mean(Xctr))
	S<-T/(n-1)
	k=0
	j=1
	Vj<-list()
	W=0
	for(i in g.levels){
		tmp=which(group==i)
		#Only the reduction is needed here
		x1=scale(Xctr[tmp,],center=TRUE,scale=FALSE)
		tmpx=x1-mean(x1)
		Wtmp<-t(tmpx)%*%tmpx
		Vj[[j]]<-(t(tmpx)%*%tmpx)/(length(tmp)-1)
		
		if(k!=0){
			W<-W+t(tmpx)%*%tmpx
		}else{
			W<-t(tmpx)%*%tmpx
			k=1
		}
		j=j+1
	}
	V<-W/(n-nb.g)
	B<-T-W
	A<-B/(nb.g-1)
	# return(list(A=A,B=B,T=T,W=W,V=V))
	# The line bellow lead to complex number, was changed to the next three
	# lines
	tmp<-eigen(solve(V)%*%round(A,dig=10))
	ldaTest$canEigval<-tmp$values
	U<-tmp$vectors
	#eigenvectors
	C<-(t(U)%*%V%*%U)
	C[is.na(C)]<-0
	C<-round(C,digits=8)^(1/2)
	dC<-diag(C)
	C<-matrix(0,nrow(C),ncol(C))
	diag(C)<-dC
	
	
	#Calculates the C matrix "Final vectors for the explanatory variables"
	ldaTest$C<-U%*%solve(C)
	colnames(ldaTest$C)<-paste("Disc.ftn", 1:m,sep="")
	rownames(ldaTest$C)<-Var.names
	#Calculates the final position of each object 
	ldaTest$F.model<-Xctr%*%ldaTest$C
	colnames(ldaTest$F.model)<-paste("Disc.ftn", 1:m,sep="")
	rownames(ldaTest$F.model)<-Obj.names[sel]
	
	ldaTest$F.predict<-X.cor.notsel%*%ldaTest$C
	colnames(ldaTest$F.predict)<-paste("Disc.ftn", 1:m,sep="")
	rownames(ldaTest$F.predict)<-Obj.names[notsel]
	
		
	# Test for the hypothesis of homogeneity of the within group dispersion 
	# matrices
	#Thestatistics 
	mrk=1
	ldaTest$Stats<-list()
	for(i in 1:nb.g){
		nj=length(which(group==g.levels[i]))
		if(det(Vj[[i]])==0){
			mrk=0
			cat("Testing for the homogeneity within group could not be done\n")
			break
		}
		if(i==1)
			Homo.Chi<-(nj-1)/2*log(det(V)/det(Vj[[i]]))
		else
			Homo.Chi<-Homo.Chi+(nj-1)/2*log(det(V)/det(Vj[[i]]))
	}
	if(mrk==1){
		dfreedom=(nb.g-1)*m*(m+1)/2
		Homo.Chi.Prob=pchisq(Homo.Chi,dfreedom,lower.tail=FALSE)
		if(print.res==TRUE){
			cat("\nStatistical results\n")
			cat('Homogeneity Chi-Square = ',Homo.Chi,'  DF = ', dfreedom,
				'  Prob = ',Homo.Chi.Prob,'\n',sep="")
		}else{}
		ldaTest$Stats$Homo=c(Homo.Chi, dfreedom, Homo.Chi.Prob)
		names(ldaTest$Stats$Homo)<-c("X2.Value", "DF", "Prob")
	}else{
		ldaTest$Stats$Homo=c("Could not be compute")
	}
	# Find the classification functions
	Vinv=solve(V)
	class.ftn<-matrix(NA,nb.g,m+1)
	for(i in 1:nb.g){
		class.ftn[i,1]<--0.5*(t(centroid[,i])%*%Vinv%*%centroid[,i])
		class.ftn[i,2:(m+1)]<-Vinv%*%centroid[,i]
	}
	ldaTest$Class.Ftn<-class.ftn
	rownames(ldaTest$Class.Ftn)<-paste("Group", g.levels)
	colnames(ldaTest$Class.Ftn)<-c("Constant",paste("Weight",1:m))

	# Reassign all or the unselected objects to every group and keep the highest 
	# scores
	nn<-length(notsel)
	tmp<-t(ldaTest$Class.Ftn);tmp0=cbind(rep(1,nn),X.cor.notsel)
	ldaTest$Scores<-do.call("rbind",lapply(1:nn,function(y) 
		apply((tmp)*tmp0[y,],2,sum)))
#	colnames(ldaTest$Scores)<-paste("Disc.ftn", 1:m,sep="")
	rownames(ldaTest$Scores)<-Obj.names[notsel]
	ldaTest$Grp.Mod<-group
	ldaTest$Grp.Obs<-group.notsel
	ldaTest$Grp.Ass<-as.factor(apply(ldaTest$Scores,1,function(y) 
		g.levels[which.max(y)]))
	
	#Find the group centroids of the data used to build the model
	ldaTest$Grp.Ctr.Mod<-do.call("cbind",lapply(g.levels,function(y) 
		apply(ldaTest$F.model[which(group==y),],2,mean)))
	colnames(ldaTest$Grp.Ctr.Mod)<-paste("Group", g.levels)
	rownames(ldaTest$Grp.Ctr.Mod)<-paste(Var.names,".Ctr", sep="")
	#Find the group centroids of the data used to test the model, based on the 
	#observed groupments
	ldaTest$Grp.Ctr.Obs<-do.call("cbind",lapply(g.levels,function(y) 
		apply(ldaTest$F.predict[which(group.notsel==y),],2,mean)))
	colnames(ldaTest$Grp.Ctr.Obs)<-paste("Group", g.levels)
	rownames(ldaTest$Grp.Ctr.Obs)<-paste(Var.names,".Ctr", sep="")
	#Find the group centroids of the data used to test the model, based on the 
	#observed groupments
	ldaTest$Grp.Ctr.Ass<-do.call("cbind",lapply(g.levels,function(y) 
		apply(ldaTest$F.predict[which(ldaTest$Grp.Ass==y),],2,mean)))
	colnames(ldaTest$Grp.Ctr.Ass)<-paste("Group", g.levels)
	rownames(ldaTest$Grp.Ctr.Ass)<-paste(Var.names,".Ctr", sep="")

	Table<-table(ldaTest$Grp.Obs,ldaTest$Grp.Ass)
	p.c.c<-round(sum(diag(Table))/sum(Table)*100,dig=1)
	Total<-apply(Table,1,sum)
	P.correct<-round(diag(Table)/apply(Table,1,sum)*100,dig=1)
	Table<-cbind(Table,Total,P.correct)
	Total<-c(apply(Table[1:nb.g,1:nb.g],2,sum),sum(Total),p.c.c)
	ldaTest$Class.Table<-rbind(Table,Total)
	
	# Create the statistic matrix
	L=tail(cumprod(1/(1+	ldaTest$canEigval)),n=1)
	mat.tmp<-matrix(NA,m,3)
	dfreedom=m*(nb.g-1)
	mat.tmp[1,1]=-1*((n-1)-1/2*(m+nb.g))*log(L)
	mat.tmp[1,2]=dfreedom
	mat.tmp[1,3]=pchisq(mat.tmp[1,1],dfreedom,lower.tail=FALSE)
	if(print.res==TRUE){
		cat('Disc.Function ', 1,'-',m, ': Chi-Square = ',mat.tmp[1,1],
			'  DF = ', mat.tmp[1,2],'  Prob = ', mat.tmp[1,3],'\n',sep="")
	}else{}
	for(i in 2:m){
		k=i-1
		mat.tmp[i,2]=(m-k)*(nb.g-k-1)
		L=tail(cumprod(1+	ldaTest$canEigval[i:m]),n=1)
		mat.tmp[i,1]=((n-1)-1/2*(m+nb.g))*log(L)
		if(mat.tmp[i,2]>=1)
			tmp=pchisq(mat.tmp[i,1],mat.tmp[i,2],lower.tail=FALSE)
		else
			tmp=NA
		if(print.res==TRUE){
			cat('Disc.Function ', i,'-',m, ': Chi-Square = ',
				mat.tmp[i,1],'  DF = ', mat.tmp[i,2],'  Prob = ',
				tmp,'\n',sep="")
		}else{}
		mat.tmp[i,3]=tmp
	}
	
	colnames(mat.tmp)<-c("X2.Value","DF","Probability")
	rownames(mat.tmp)<-c(paste("Disc.Function ", 1:m,"-",m,sep=""))
	mat.tmp[,1]<-round(mat.tmp[,1],digits=2)
	mat.tmp[,3]<-round(mat.tmp[,3],digits=4)
	
	if(is.matrix(mat.tmp)){
		tmp=length(which(mat.tmp[,3]<=alpha))
		if(print.res==TRUE){
			if(tmp>1){
				cat("\nThe first", tmp, "axes are found to be significative\n") 
			}else{
				if(tmp==1){
					cat("\nOnly the first axis is found to be significative\n") 
				}else{
					cat("\nNone of the axis tested is significative\n") 
				}
			}
		}	
	}
	ldaTest$Stats$Axes<-mat.tmp
	return(ldaTest)
}