"rdaTest" <-
function(YY.mat, XX.mat, which.cov=NULL, scale.Y=FALSE, testF=NULL, 
	nperm=999, print.res=TRUE, print.cum=FALSE)

# DESCRIPTION

# Function RDA for simple or partial RDA, with permutation tests, computed as
# described in Numerical Ecology, Chapter 11 (Legendre & Legendre 1998).
#
# Pierre Legendre & Sbastien Durand, Universit de Montral, March-April 2005
#
#
# PARAMETERS:
#
# YY.mat 		(nxp) is the site-by-species data table
# XX.mat 		(nxm) contains the explanatory variables. The number of
#				variables 'm' is computed after eliminating collinear
#				explanatory variables, if any. Covariables, if any, are also  
#               found in that table.
# which.cov     A vector (1xmax(m-1)) defining the covariables by there column 
#               numbers in table XX.mat. 
# scale.Y       contains a logical value: should YY.mat be standardized, or not
# testF			When NULL, the program will ask the user if he/she wishes to 
#				test the F statistic. If testF is TRUE or FALSE, no question
#				will be asked; the program will perform the test, or not, in
#				accordance with that indication.
# nperm			number of permutations for the F test. If NULL, a question 	
#				will be asked.
# print.res 	prints most of the rdaTest output on the screen.
# print.cum 	prints the fractions of the response variable's (e.g. species)
#				variances explained by canonical axes 1, 2, 3, ... and by the
#				whole canonical analysis.
#
#
# The function returns an output list containing the following ELEMENTS:
# return(list(VIF=vif.res, canEigval=canEigval, U=U, USc2=UL, F=F, Z=Z,
#	FSc2=FSc2, ZSc2=ZSc2, biplotScores1=posX, biplotScores2=posXSc2,
#	FitSpe=Frac$rdaFitSpe, VarExpl=Frac$VarExpl, X.mat=X.mat, AxisVar)
#
# OUTPUT
#
# VIF			variance inflation factors for the explanatory variables X and 
#				not	on the covariable defined by which.cov; the value is 0 for
#				entirely collinear variables 
# canEigval     canonical eigenvalues
# U (pxk)		canonical eigenvectors normalized to 1 (scaling 1)
# USc2 (pxk)	canonical eigenvectors normalized to sqrt(eigenvalue) 
#				(scaling 2)
# F (nxk)		matrix of object scores (scaling 1)
# Z (nxk)		matrix of fitted object scores (scaling 1)
# FSc2			matrix of object scores (scaling 2)
# ZSc2			matrix of fitted object scores (scaling 2)
# biplotScores1	biplot scores of explanatory variables (scaling 1)
# biplotScores2	biplot scores of explanatory variables (scaling 2)
# FitSpe		table of cumulative fit per species (in %) as fraction of
#				variance of species
# VarExpl		vector of total % fit per species after all canonical axes 
# X.mat			original X matrix (required by the plotting function)
# AxisVar		Eigenvalues as in Canoco: fraction of variance of YY.mat 
#               explained by each canonical axis.
{
	library(MASS)
	if(is.logical(scale.Y)){
	}else{
		stop("Wrong operator; 'center.Y' should be either 'FALSE' or 'TRUE'")	
	}
	# Read the data tables. Transform them into matrices Y, X, and W
	Y.mat=as.matrix(YY.mat)
	X.mat=as.matrix(XX.mat)
	sitenames<-rownames(X.mat)
	if(is.null(rownames(X.mat)) & is.null(rownames(Y.mat))){
		sitenames<-paste("Site",1:nrow(X.mat),sep="")
	}else{
		if(is.null(rownames(X.mat))){
			sitenames<-rownames(Y.mat)
		}else{
			sitenames<-rownames(X.mat)
		}
	}
	n=nrow(Y.mat)
	p=ncol(Y.mat)
	Y = apply(Y.mat,2,scale,center=TRUE,scale=scale.Y)
	rownames(Y)<-sitenames
	X = apply(X.mat,2,scale,center=TRUE,scale=TRUE)
	tmp=colnames(X)
	q=0
	if(length(which.cov)!=0){
		covar=TRUE
		W.mat=as.matrix(X.mat[,which.cov])
		X=as.matrix(X.mat[,-which.cov])
		colnames(X)=tmp[-which.cov]
		q=ncol(W.mat)
		W=apply(W.mat,2,scale,center=TRUE,scale=TRUE)
		if(print.res==TRUE) cat("\nThere is a matrix of covariables\n")
	}
	else{
		covar=FALSE
		if(print.res==TRUE) cat("\nThere is no matrix of covariables\n")
	}
	if(ncol(X) == 0) stop("Matrix X contains no variable")

	# Identify collinear columns in X and remove them
	vif.res=vif(X,print.vif=print.res)
	if(length(which(vif.res==0))!=0)
		X.cor = X[,-which(vif.res==0)]
	else
		X.cor = X
	m=ncol(X.cor)
	if(print.res==TRUE){ 
		cat('\n')
		cat('Variance Inflation Factors (VIF)','\n')
		print(vif.res)
	}

    # Construct an identity matrix "Inn" of size (nxn)
	Inn <- diag(n)
	# In covariables present, regress X on W by QR decomposition. Obtain X.res
	if(covar==TRUE) {
        Q.W = qr(W, tol=1e-6)                 #   invW = ginv(t(W) %*% W)
		projW = qr.fitted(Q.W, Inn)           #   projW = W %*% invW %*% t(W)
		X.res = X.cor - projW %*% X.cor
	# ... and regress Y on X and W through QR decomposition
		XW=cbind(X.cor,W)
		Q.XW = qr(XW, tol=1e-6)               #   invXW = ginv(t(XW) %*% XW)
		projXW = qr.fitted(Q.XW, Inn)         #   projXW = XW %*% invXW %*% t(XW)
		XW=cbind(X.cor,W)
		XW.fit = projXW %*% Y
		SS.Yfit.XW = sum(XW.fit*XW.fit)
	}else{
		X.res = X.cor
	}

	# Compute projector of X and Yfit.X by QR decomposition
	Q.X = qr(X.res, tol=1e-6)                 #   invX = ginv(t(X.res) %*% X.res)
	projX = qr.fitted(Q.X, Inn)               #   projX = X.res %*% invX %*% t(X.res)
	Yfit.X = projX %*% Y
	rownames(Yfit.X)<-sitenames
	
	# Compute R-square and adjusted R-square
	SS.Y = sum(Y*Y)
	SS.Yfit.X = sum(Yfit.X*Yfit.X)
	
	# SS.Y = sum(diag(cov(Y)))
	# SS.Yfit.X = sum(diag(cov(Yfit.X)))
	Rsquare=SS.Yfit.X/SS.Y
	if(covar==FALSE) {
		totalDF=n-1
		residualDF=n-m-1
		adjRsq = 1-((1-Rsquare)*totalDF/residualDF)
	} else { adjRsq = 0 }

	# Test significance of the canonical F statistic
	testFF="N"
	if(is.null(testF)){
		cat('\n')
		cat("Test the bimultivariate F-statistic? Yes = 'Y', No = 'N'\n")
		testFF=toupper(scan(file="",what="character",nlines=1,quiet=TRUE))
	} else { if(testF==TRUE) testFF="Y" }

	if(testFF=="Y") {
		if(is.null(nperm)){
			cat('\n','How many permutations? Ex. 499, 999, ...','\n',sep="")
			nper=toupper(scan(file="",what="integer",nlines=1,quiet=TRUE))
			nperm=as.integer(nper)
		}else{}
	}
	if(print.res==TRUE){
		cat('\n','----------','\n','\n',sep="")
		cat('Bimultivariate redundancy statistic','\n')
		cat('R-square =',Rsquare)
		if(covar==FALSE) cat('    adjusted R-square = ',adjRsq,'\n',sep="")
	}else{}

	if(testFF=="Y") {prob<-probFrda(Y,X,n,p,m,q,nperm,projX,projW,projXW,SS.Y,SS.Yfit.X,
	                      covar,SS.Yfit.XW,print.res=print.res)
	}else{
		prob="Not tested"
	}
	# PCA portion of RDA: eigenanalysis, then compute the F and Z matrices, etc.
	SS.Y=SS.Y/(n-1)
	Yhat.cov = cov(Yfit.X)
	Yhat.eig = eigen(Yhat.cov)
	Yhat.val=Yhat.eig$values
	axenames<-paste("Axis",1:length(Yhat.val),sep="")
	names(Yhat.val)<-axenames
	Yhat.vec=Yhat.eig$vectors
	rownom<-colnames(X.res,do.NULL = FALSE, prefix = "m")
	colnames(Yhat.vec)<-axenames
	rownames(Yhat.vec)<-colnames(Y,do.NULL = FALSE, prefix = "p")
	# How many canonical eigenvalues?
	k=0
	for(i in 1:min((n-1),p,m)) { if(Yhat.val[i] > 0.00000001) k=k+1 }
	if(k == 0) stop("No variation in X after controlling for covariables")
	#
	canEigval=Yhat.val[1:k]
	EigvalCanoco = 100*canEigval/SS.Y
	cumulVar=vector(mode="numeric",k)
	cumulVar[1]=EigvalCanoco[1]
	if(k > 1) {
		for(j in 2:k) { cumulVar[j]=cumulVar[j-1]+EigvalCanoco[j] } 
	}
	
	# for (scaling==1)
	U = as.matrix(Yhat.vec[,1:k])
	colnames(U)<-axenames[1:k]
	F = Y %*% U
	Z = Yfit.X %*% U
	
	# for (scaling==2)
	lambdaSc2=vector(mode="numeric",k)
	for(j in 1:k) {
		lambdaSc2[j]=sqrt(Yhat.val[j])
	}
	if(k == 1) {
		UL=U * lambdaSc2[1]
		FSc2 = F * (1/lambdaSc2[1])
		ZSc2 = Z * (1/lambdaSc2[1])
	}
	if(k > 1) {
		UL=U %*% diag(lambdaSc2)
	#	colnames(UL)<-axenames[1:k]
		FSc2 = F %*% diag(1/lambdaSc2)
	#	colnames(FSc2)<-axenames[1:k]
		ZSc2 = Z %*% diag(1/lambdaSc2)
	#	colnames(ZSc2)<-axenames[1:k]
	}
	# Compute the 'Biplot scores of environmental variables' --
	# First, compute the correlations between X and Z, or X and ZSc2
	# Positions for scaling 1: compute the correlations...
	corXZ=cor(X,Z)
	
	# ...then, weigh these correlations
	# by the diagonal matrix 'D' of weights  sqrt(lambda(k)/SS.Y)
	tmp<-(sqrt(Yhat.eig$values[1:k]/SS.Y))
	if(length(tmp)!=1){
		D = diag(tmp)
		posX = corXZ %*% D
		colnames(posX)<-axenames[1:k]
	}else{
		posX=corXZ * tmp
	}
	# Positions for scaling 2:
	posXSc2=corXZ   
	# which is cor(X,Z); posXSc2=cor(X,ZSc2) produces the same result
	
	# Print results
	if(print.res==TRUE){
		cat('\n','Number of objects:  n = ',n,'\n',sep="")
		cat('Number of response variables in Y:  p =',p,'\n')
		cat('Number of explanatory variables in X:  m =',m,
		    ' after removing covariables and collinear X variables, if any','\n')
		if(covar==TRUE) cat('Number of covariables in W:  q =',q,'\n')
		cat('Number of canonical eigenvalues:  k =',k,'\n')
		cat('\n')
		cat('Total variance =',SS.Y,'\n','\n')
		cat('Eigenvalues','\n')
		cat(canEigval,'\n','\n')
		cat('Relative eigenvalues (% variance)','\n')
		cat(EigvalCanoco,'\n','\n')	
		cat('Cumulative % variance of species data','\n')
		cat(cumulVar,'\n','\n')
	}else{}
	
	# Compute the "Cumulative fit per species as fraction of variance of 
	# species" table
	Frac=FractionBySpecies(Y.mat,UL,n,p,k,print.cum=print.cum)

	# Create the output list containing the following elements:
	#
	# VIF=vif.res: Variance Inflation Factors (VIF)
	# canEigval=canEigval: vector of canonical eigenvalues
	# U=U: canonical eigenvectors normalized to 1 (scaling 1)
	# USc2=UL: canonical eigenvectors normalized to sqrt(eigenvalue) (scaling 2)
	# F=F: matrix of object scores (scaling 1)
	# Z=Z: matrix of fitted object scores (scaling 1)
	# FSc2=FSc2: matrix of object scores (scaling 2)
	# ZSc2=ZSc2: matrix of fitted object scores (scaling 2)
	# biplotScores1=posX: biplot scores of explanatory variables (scaling 1)
	# biplotScores2=posXSc2: biplot scores of explanatory variables (scaling 2)
	# FitSpe=Frac$rdaFitSpe: table of cumulative fit per species (in %) 
	#    as fraction of variance of species
	# VarExpl=Frac$VarExpl: vector of total % fit per species after all 
	# canonical axes 
	# ProbFrda=prob: probability associated with F test of the canonical 
	# relationship
	# X.mat=X.mat: original X matrix (required by the plotting function)

	return(list(VIF=vif.res, canEigval=canEigval, U=U, USc2=UL, F=F, Z=Z, 
			FSc2=FSc2, ZSc2=ZSc2, biplotScores1=posX, biplotScores2=posXSc2, 
			FitSpe=Frac$rdaFitSpe, Rsquare=Rsquare, Rsquare.adj=adjRsq,
			VarExpl=Frac$VarExpl, ProbFrda=prob, X.mat=X.mat,
			AxisVar=EigvalCanoco))
}